#ifndef UART_APP_H
#define UART_APP_H

#include "uart_register.h"


#define RX_BUFF_SIZE    0x100
#define TX_BUFF_SIZE    100

typedef enum {
    FIVE_BITS = 0x0,
    SIX_BITS = 0x1,
    SEVEN_BITS = 0x2,
    EIGHT_BITS = 0x3
} UartBitsNum4Char;

typedef enum {
    ONE_STOP_BIT             = 0,
    ONE_HALF_STOP_BIT        = BIT2,
    TWO_STOP_BIT             = BIT2
} UartStopBitsNum;

typedef enum {
    NONE_BITS = 0,
    ODD_BITS   = 0,
    EVEN_BITS = BIT4
} UartParityMode;

typedef enum {
    STICK_PARITY_DIS   = 0,
    STICK_PARITY_EN    = BIT3 | BIT5
} UartExistParity;

typedef enum {
    BIT_RATE_9600     = 9600,
    BIT_RATE_19200   = 19200,
    BIT_RATE_38400   = 38400,
    BIT_RATE_57600   = 57600,
    BIT_RATE_74880   = 74880,
    BIT_RATE_115200 = 115200,
    BIT_RATE_230400 = 230400,
    BIT_RATE_460800 = 460800,
    BIT_RATE_921600 = 921600
} UartBaudRate;

typedef enum {
    NONE_CTRL,
    HARDWARE_CTRL,
    XON_XOFF_CTRL
} UartFlowCtrl;


typedef enum {
    EMPTY,
    UNDER_WRITE,
    WRITE_OVER
} RcvMsgBuffState;

typedef struct {
    uint32     RcvBuffSize;
    uint8     *pRcvMsgBuff;
    uint8     *pWritePos;
    uint8     *pReadPos;
    uint8      TrigLvl; //JLU: may need to pad
    RcvMsgBuffState  BuffState;
} RcvMsgBuff;

typedef struct {
    uint32   TrxBuffSize;
    uint8   *pTrxBuff;
} TrxMsgBuff;

typedef enum {
    BAUD_RATE_DET,
    WAIT_SYNC_FRM,
    SRCH_MSG_HEAD,
    RCV_MSG_BODY,
    RCV_ESC_CHAR,
} RcvMsgState;

typedef struct {
    UartBaudRate 	     baud_rate;
    UartBitsNum4Char  data_bits;
    UartExistParity      exist_parity;
    UartParityMode 	    parity;    // chip size in byte
    UartStopBitsNum   stop_bits;
    UartFlowCtrl         flow_ctrl;
    RcvMsgBuff          rcv_buff;
    TrxMsgBuff           trx_buff;
    RcvMsgState        rcv_state;
    int                      received;
    int                      buff_uart_no;  //indicate which uart use tx/rx buffer
} UartDevice;

typedef struct {
	uint32_t 			uart_BaudRate;
	UartStopBitsNum 	uart_StopBit;
	UartParityMode 		uart_ParityBit;
} uartcfg_t;

void uart0_init(void);
void ICACHE_FLASH_ATTR UART0_SendByte(uint8_t data);
void ICACHE_FLASH_ATTR UART0_SendBytes(uint8_t *pData, unsigned int size);
/*
 * Call to check the current available space in the tx buffer
 */
uint16_t ICACHE_FLASH_ATTR UART0_TxAvailableSpace(void);
/*
 * Call to return the number of byte in the rx buffer
 */
uint16_t ICACHE_FLASH_ATTR UART0_RxAvailable(void);
/*
/*
 * Call to return the Rx data from the buffer, return the actual byte return.
 */
uint16_t ICACHE_FLASH_ATTR UART0_RxReturn(uint8_t* pData, uint16_t size);

/*
 * Insert default configuration into the syscfg
 */
void ICACHE_FLASH_ATTR UART_Defult_Cfg(uartcfg_t *uartcfg);



#endif

