/*
 *  Disclaimer and terms of usage
 *  -----------------------------
 *  Redistribution and use in source and binary forms, with or without modification,
 *  are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *  THE POSSIBILITY OF SUCH DAMAGE.
*/
/*
 * DriverMgt.h
 *
 *  Created on: Apr 8, 2016
 *      Author: khgoh
 */

#ifndef DRIVERMGR_H_
#define DRIVERMGR_H_

#include "globe.h"
#include <inttypes.h>
/*
 * Call once during startup
 */
void Dmgr_Init(unsigned int (*pSendCmdCallBack)(char *pData,unsigned int size));

/*
 * register the i2c driver, on success, it will return the drivermgrSequence number
 * will return -1 if fail to register.
 */
int DMgr_RegisterDriver(uint8_t i2cid,
		void (*refreshCB)(clientprop_t*), //call once in the fix interval
		void (*replyCB)(clientprop_t*,char **token, int tokenCnt), // call when remote client reply
		void (*cmdtimeoutCB)(clientprop_t*), //call when received timeout notice
		void (*remoteClientDisconnectedCB)(clientprop_t*), //call when remote client disconnected
		void (*resetAllCB)(clientprop_t*));	//call when the wifi server is disconnected.

/*
 * will do the callback on each remote client with the drivermgrsequence
 * must call this function every second.
 */
void DriverRefresh(void);

/*
 * Call during remote client reply command.
 */
void DMgr_ClientReply(char **token, int tokenCnt);
/*
 * Call when received TIMEOUT message from remote client
 */
void DMgr_ClientCmdTimeout(clientprop_t* pClientProp);


/*
 * Call when client just online
 */
void DMgr_ClientOnline(clientprop_t* pClientProp);

/*
 * Call when client just offline
 */
void DMgr_ClientOffline(clientprop_t* pClientProp);

/*
 * return true is current process i2c device is online
 * Only allow to call this function during driver callback.
 */
int DMgr_isDeviceOnline(void);
void DMgr_SetDeviceOnline(void);
void DMgr_SetDeviceOffline(void);

/*
 * Create (return null if fail), get and release I2c Device properties storage area under
 * the connected remote Client
 */
void *DMgr_CreateDevicePropertiesStorage(int storageSize);
void DMgr_ReleaseDevicePropertiesStorage(clientprop_t *pClientProp, int driverID);
void *DMgr_GetDevicePropertiestStorage(void);
/*
 * request to send the command string.
 * return true if able to insert the cmdString into the send buffer.
 */
int DMgr_DriverSend(char *cmdString, char *attbString);


#define DMgr_RecNextStep(pClientProp,pRefreshFunc) do {\
		(pClientProp)->i2ccmdstep++;\
		(pClientProp)->i2cPandingReply=0;\
		(*(pRefreshFunc))((pClientProp));\
	}while(0)

#define DMgr_SendDone(pClientProp) do {\
		(pClientProp)->i2ccmdstep=stepdone;\
		(pClientProp)->i2cPandingReply=0;\
	}while(0)
/*
 * Return Device Properties structure. App Require to Cast the return pointer
 */
void *DMgr_GetDeviceProperties(int remoteClientRef, int DeviceID);

/*
 * Call when server disconnected
 */
void DMgr_Serverdisconnected(void);

#endif /* DRIVERMGR_H_ */
