/*
 *  Disclaimer and terms of usage
 *  -----------------------------
 *  Redistribution and use in source and binary forms, with or without modification,
 *  are permitted provided that the following conditions are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *  THE POSSIBILITY OF SUCH DAMAGE.
*/
/*
 * PollMgr.h
 *
 *  Created on: Jul 9, 2015
 *      Author: khgoh
 */

#ifndef INC_POLLMGR_H_
#define INC_POLLMGR_H_
#include <signal.h>
#include <poll.h>
/*
 * call to insert a fd for poll sequence
 * name of the fd max char is 20char+1null.
 * if name is null, it will fill in with the fd.
 * return true if successful inserted into the list
 * The Call back function will have format at below,
 * void CallBackFunc(int fd, short int event, void *ptr);
 * fd - The file description
 * event - the trigger event
 * ptr - pointer provided during insert will pass on to the callback.
 */
int PollMgr_Insert(char *name,int fd, short int event, void (*cb)(int,short int,void*),void *ptr);
/*
 * remove the poll fd from the list
 */
void PollMgr_Remove(int fd);

/*
 * Call from Main program loop routine.
 * Each call will cause the system to sleep/block for PollSleep_ms
 * Default PollSlee_ms is 10ms
 * If the fd is trigger, it will do the respective callback.
 */
void PollMgr_Refresh(void);

//to change the internal setting of PollSleep_ms
//#define PollSleep_ms 10	//time to sleep on each block.

#endif /* INC_POLLMGR_H_ */
